
% Example of AM modulation and demodulation techniques with figures
  clear all; close all;
  
  mod = 5;             % AM type: 1=DSP-C, 2=DSB-CS, 3=SSB-R=USB, 4=SSB-L=LSB, 5=test
  fc = 6000;           % AM carrier frequency 
  dA = 0.5;            % AM modulation depth for DSB-C
  Nwin=256; Nover=Nwin-32; Nfft=2*Nwin;  % for STFT plot
  
  % Modulating signal m(t)
  [m,fs] = audioread('speech8000.wav'); % read mono audio from file
  K=5; m = resample(m,K,1); fs=K*fs;    % upsample K-times for freq-UP
  N = length(m);                        % number of signal samples
  dt = 1/fs; t = dt*(0:N-1)';           % time
  df=fs/Nfft; f=df*(-Nfft/2:Nfft/2-1);  % frequencies for STFT display
  
  figure; plot(t,m); xlabel('t [s]'); grid; title('m(t)'); pause
  figure; spectrogram(m,Nwin,Nover,f,fs,'yaxis'); title('STFT m(t)'); pause

% Create base-band signal for AM modulation of the carrier
  if(mod==1) s = (1+dA*m);                end           % DSB-C
  if(mod==2) s = m;                       end           % DSB-CS
  if(mod==3) s = m + j*imag(hilbert(m));  end           % SSB-R = USB
  if(mod==4) s = m - j*imag(hilbert(m));  end           % SSB-L = LSB
  if(mod==5) s = m + j*0.5*sin(2*pi/50*(0:N-1)'); end   % DSB-CS test of complex
  figure; spectrogram(s,Nwin,Nover,f,fs,'yaxis'); title('STFT s(t)'); pause
  
% Carrier AM modulation - frequency-up conversion of the base-band signal
% x = real( s .* exp( j*2*pi*fc*t ) );     % alternative
  x = real(s).*cos(2*pi*fc*t) - imag(s).*sin(2*pi*fc*t); 
  figure; spectrogram(x,Nwin,Nover,f,fs,'yaxis'); title('STFT UP Real'); pause

% Carrier AM demodulation - frequency-down conversion to the baseband 
% y = hilbert( x ) .* exp( -j*2*pi*fc*t );  % alternative
  y = 2*x.*cos(2*pi*fc*t) - 2*j*x.*sin(2*pi*fc*t);  
  figure; spectrogram(y,Nwin,Nover,f,fs,'yaxis'); title('STFT Down'); pause

% Ideal LowPass filter in frequency domain
  df=fs/N; M = floor(fc/df); 
  SS = fft(y); SS(M+1:N-M) = zeros(1,N-2*M); ss = ifft( SS );
  figure; spectrogram(ss,Nwin,Nover,f,fs,'yaxis'); title('STFT Filter'); pause

% ERROR after frequency UP & DOWN 
  ERROR_SPECTRUM = max( abs( fft(s) - SS ) )/N,
  ERROR_SIGNAL   = max( abs( s - ss ) ), pause

% Recovering m(t) from ss(t) 
  if(mod==1)                   mm = (abs(ss)-1)/dA; end
  if(mod==2 | mod==3 | mod==4) mm = real(ss); end
  if(mod==5)                   m  = [real(s)  imag(s) ];
                               mm = [real(ss) imag(ss)];
  end
 
  n = 500:N-500+1; t=t(n); m=m(n); mm=mm(n); % with or without it
  
  figure;
  subplot(211); plot(t,m,'r-',t,mm,'b-'); xlabel('t [s]'); title('m(t) & mm(t)');
  subplot(212); plot(t,m-mm,'b-'); xlabel('t [s]'); title('ERR signal'); grid; pause

  disp('Playing source'); soundsc(m,fs); pause
  disp('Playing demod '); soundsc(mm,fs); pause
