
% Radio FM AM-FM: FM Radio stereo MPX signal synthesis
  clear all; close all;

  fs = 192000;         % FM radio sampling frequency

% Audio signals: left (L) and right (R) channels
  [x1, fs1] = audioread('GOODBYE.WAV');
  [x2, fs2] = audioread('DANKE.WAV');
  fs1, fs2,
     figure;
     subplot(211); plot(x1); title('x1(n)');
     subplot(212); plot(x2); title('x2(n)'); pause
     figure
     subplot(211); spectrogram(x1, 512, 512-64, 512, fs1); title('STFT x1(n)'); 
     subplot(212); spectrogram(x2, 512, 512-64, 512, fs2); title('STFT x2(n)'); pause
     soundsc(x1,fs1); pause
     soundsc(x2,fs2); pause

% Up-sampling    
  [U1,D1]=rat(fs/fs1,0.001),   % up-sampling x1(n)
  x1 = resample(x1,U1,D1);
  [U2,D2]=rat(fs/fs2,0.001),   % up-sampling x2(n)
  x2 = resample(x2,U2,D2);
    figure;
    subplot(211); plot(x1); title('x1(n) up-sampled');
    subplot(212); plot(x2); title('x2(n) up-sampled'); pause
    figure
    subplot(211); spectrogram(x1, 512, 512-64, 512, fs); title('STFT x1(n) up-sampled'); 
    subplot(212); spectrogram(x2, 512, 512-64, 512, fs); title('STFT x2(n) up-sampled'); pause
    subplot(111);
    soundsc(x1,fs); pause
    soundsc(x2,fs); pause

% Signal length adjustment
  N1 = length(x1); N2=length(x2);
  Nmax = max( N1,N2 ); n = 1:Nmax;
  x1(N1+1:Nmax,1)=zeros(Nmax-N1,1);   % adjusting length of x1(n)
  x2(N2+1:Nmax,1)=zeros(Nmax-N2,1);   % adjusting length of x2(n)
% x1 = zeros(Nmax,1);                 % for tests
% x2 = zeros(Nmax,1);                 %
  df=fs/Nmax; f = df*(0:Nmax-1);      % will be use in displaying results
  ff=0:fs/10000:fs/2;                 % 

% Creation of MPX hybrid signal - AM modulation of carriers
  x12p = x1+x2;                % L+R channels
  x12m = x1-x2;                % L-R channels

% Pilot and its multiples
  dt=1/fs; t=dt*(0:Nmax-1)';
  fi = (2*pi*19000*t);
  c19 = cos(fi);              % pilot 19kHz
  c38 = cos(2*fi);            % 2x for L-R = 38kHz
  c57 = cos(3*fi);            % 3x for RDS = 57kHz

% MPX hybrid signal
  x = 0.9*x12p + 0.1*c19 + 0.9*x12m.*c38 + 0.05*c57;
% x = 0.9*x12p + 0.9*x12m.*c38;
% x = c57;
    figure; plot(f,20*log10(abs(fft(x)))); grid; xlabel('f (Hz)'); title('MPX OUT (dB)'); pause
    figure; spectrogram(x, 512, 512-64, 512, fs); title('MPX OUT'); pause

% FM Modulation
  fc = 0;                     % carrier, in BaseBand 0Hz
  fmax = 60000;               % maximum modulating frequency
  BW = 160000;                % available bandwidth, BW > 2*fmax
  df = (BW/(2*fmax)-1)*fmax,  % modulation depth
% df = 40000;                 % for tests
  beta = df/fmax,             % modulation index
  y = exp( j*2*pi*(fc*t + df*cumsum(x)*dt) );
    figure; plot(f,20*log10(abs(fft(y)))); grid; xlabel('f (Hz)'); title('FM (dB)'); pause
    figure; spectrogram(y, 512, 512-64, 512, fs); title('FM (dB/Hz)'); pause

% Complex-value signal to be sent, e,g. by ADALM-PLUTO
% Transmitter: Freq-UP from BB (0 Hz) to target frequency, e.g. 101.6 MHz
% Channel distortions: attenuation, reflections, interferences, noise
% Receiver: Freq-DOWN from target frequency to BB (0 Hz)

% Radio FM AM-FM: FM Radio stereo MPX signal analysis

% FM demodulation
  y21 = y(2:end) .* conj(y(1:end-1));     % complex values with phase difference 
  dfi = atan2( imag(y21), real(y21) );    % finding the phase shift using angle(y21) or atan2(imag(y21), real(y21))
  finst = (1/(2*pi)) * dfi/dt;            % instantaneous frequency
  xe = (finst-fc) / df;                   % minus carrier frequency, taking into account the modulation depth  
  xe = [ 0; xe];
  ERR_FM = max(abs(x(2:end)-xe(2:end))),
        figure; plot(f,20*log10(abs(fft(xe)))); grid; xlabel('f (Hz)'); title('MPX IN (dB)'); pause
        figure; spectrogram(xe, 512, 512-64, 512, fs); title('MPX IN (dB/Hz)'); pause
        figure; plot(n(2:end),x(2:end),'r-',n(2:end),xe(2:end),'b-'); grid; title('x(n) and xe(n)'); pause

% Restoration of x12p=L+R signal: Low-Pass filtering
  M = 500;
  hLP = fir1(M,[16000]/(fs/2),'low',chebwin(M+1,120)); % odzysk L+P
        figure; plot(ff,20*log10(abs(freqz(hLP,1,ff,fs)))); grid; xlabel('f (Hz)'); title('|H_{LP}(f)| (dB)'); pause
  x12pR = filtfilt(hLP,1,xe);
% df=fs/Nmax; M=floor(16000/df); mask=zeros(Nmax,1); % ideal freq filter
% mask(1:M+1)=ones(M+1,1); mask(Nmax-M+1:Nmax,1)= ones(M,1);
% x12pR = ifft( fft(xe) .* mask );
  x12pR = x12pR / 0.9;          % scaling
  soundsc(x12pR,fs);            % playing
  ERR_12p = max(abs(x12p(2:end)-x12pR(2:end))),
        figure; plot(f,20*log10(abs(fft(x12p))),'r-',f,20*log10(abs(fft(x12pR))),'b-'); grid; xlabel('f (Hz)'); title('|X12pR(f)| (dB)'); pause
        figure; plot(n,x12p,'r-',n,x12pR,'b-'); grid; title('x12pR(n)'); pause

% Restoration of x12m=L-R signal: Freq-DOWN and Low-Pass filtering
  c38R = c38;                          % carrier of (L-R) signal 
  x12mR = 2 * xe .* c38R;              % frequency DOWN conversion"
  x12mR = filtfilt(hLP,1,x12mR);       % no-delay digital filter
% x12mR = ifft( fft(x12mR) .* mask );  % ideal filter in freq domain
  x12mR = x12mR / 0.9;                 % scaling
  ERR_12m = max(abs(x12m(2:end)-x12mR(2:end))),
        figure; plot(f,20*log10(abs(fft(x12mR)))); grid; xlabel('f (Hz)'); title('|X12mR(f)|+FreqDown+Filter (dB)'); pause
        figure; plot(n,x12m,'r-',n,x12mR,'b-'); grid; title('x12mR(n)'); pause
 
% Restoration of L and R from (L+R) and (L-R)        
  x1R = (x12pR + x12mR)/2;
  x2R = (x12pR - x12mR)/2;
  n = 1:length(x1);
  nR = 1:length(x1R);
  ERR_x1 = max(abs(x1-x1R)), ERR_x2 = max(abs(x2-x2R)), 
        figure;  plot(n,x1,'r-',nR,x1R,'b-'); grid; title('x1(n)'); pause
        figure;  plot(n,x2,'r-',nR,x2R,'b-'); grid; title('x2(n)'); pause

  soundsc(x1R,fs); pause % playing reconstructed L channel
  soundsc(x2R,fs); pause % playing reconstructed R channel
