function hpsf = rcosdesignTZ(BETA,SPAN,SPS,SHAPE)
% Rasied cosine (RC) and squre-root raised cosine (SRRC) FIR pulse-shaping filter design
%
% INPUT:
% BETA - rolloff
% SPAN - number of symbols per PSF
% SPS  - number of samples per symbol
% SHAPE - 'normal' (RC, raised-cosine) or 'sqrt' (SQRT, square-root raised cosine)
if(0) % TEST ONLY
   BETA = 0.35; SPAN = 8; SPS = 6; SHAPE='normal'; % input values of rcosdesign()
end

R = BETA; Nsps = SPS;         % new names       
Npsf = SPAN*SPS;              % filter length
n = 0 : Npsf; Nc = Npsf/2;    % indexes of filter samples, index of the central sample
if( isequal( SHAPE, 'normal') ) % definition of raised cosine
    hpsf = sin(pi*(n-Nc)/Nsps) ./ (pi*(n-Nc)/Nsps) .* ...
           cos(pi*R*(n-Nc)/Nsps) ./ (1-(2*R*(n-Nc)/Nsps).^2);
    hpsf(Nc+1) = 1;   
    n0 = find( abs(1-(2*R*(n-Nc)/Nsps).^2 ) < sqrt(eps) );
    hpsf(n0) = R/2 * sin(pi/(2*R));
elseif( isequal( SHAPE, 'sqrt') ) % definition of sqrt of raised cosine
    n0 = find( abs(1-(4*R*(n-Nc)/Nsps).^2) < 5*eps );
    if( prod(size(n0)) > 0 ) disp('Div by 0. Change parameters'); end
    hpsf = 4*R/pi * ( cos(pi*(1+R)*(n-Nc)/Nsps)+(Nsps./(4*R*(n-Nc)) ) ...
           .* sin(pi*(1-R)*(n-Nc)/Nsps) ) ./ (1-(4*R*(n-Nc)/Nsps).^2);
    hpsf(Nc+1) = (1+R*(4/pi-1));   
end

if(0) % Comparison with Matlab, appropriate scaling should be added
      % Change name of this function
   hpsf0 = rcosdesign(BETA,SPAN,SPS,SHAPE); % MATLAB function
   sc = hpsf0(Npsf/2)/hpsf(Npsf/2),
   plot(1:length(hpsf0),hpsf0,'ro-',1:length(hpsf),sc*hpsf,'bx-');
   error = max(abs(hpsf0-sc*hpsf)), pause
end
